namespace AJAX
{
    // classe InformazioniAjax,
    //	gestisce i dati di ingresso della pagina
    //  ed il filtro per la stessa al fine
    //  di creare un report testuale.
    public class InformazioniAjax
    {
        // variabile privata tipo stringa contenente l'output
        private System.String __inizioReport;

        // variabile privata di assegnazione Response
        private System.Web.HttpResponse __risposta;

        // costruttore, memorizza il response 
        // e crea la stringa con il dump delle variabili
        // inviate in GET o POST tramite la variabile Request
        public InformazioniAjax(
            System.Web.HttpRequest richiesta,
            System.Web.HttpResponse risposta
        )
        {
            __risposta = risposta;
            __inizioReport =
		    "REPORT DEL " +
            System.DateTime.Now.ToString() + "\r\n\r\n" + 
		    "[VARIABILI GET]\r\n" +
            this.__Dump(richiesta.QueryString) + 
		    "\r\n" +
		    "---------------------------------\r\n\r\n" + 
		    "[VARIABILI POST]\r\n" +
            this.__Dump(richiesta.Form) + 
		    "\r\n" + 
		    "_________________________________\r\n\r\n" + 
		    "[OUTPUT GENERATO]\r\n";
        }

        // metodo pubblico, inizializza la variabile
        // da usare come filtro, creata tramite la classe
        // FiltroRisposta, aggiunge a inizio file
        // leinformazioni GET e POST della variabile __inizioReport
        // ed assegna il filtro alla pagina.
        // Accetta il nome del file sul quale salvare 
        // le informazioni.
        public void CreaReport(System.String fileReport)
        {
            FiltroRisposta fr = new FiltroRisposta(__risposta.Filter, ref fileReport);

            // prima di assegnare il filtro, scrivo il report sulla Request
            // effettuato nel costruttore ed assegnato a __inizioReport
            fr.ScriviInizioReport(ref __inizioReport);

            // ora  possibile assegnare il filtro
            __risposta.Filter = fr;
        }

        // overload del metodo CreaReport
        // se non viene specificata la stringa
        // crea il file report nella cartella App_Data
        // ovvero dove si dovrebbe trovare questo stesso file
        public void CreaReport()
        {
            this.CreaReport(
                System.AppDomain.CurrentDomain.BaseDirectory +
                "\\App_Data\\report.txt"
            );
        }

        // metodo privato, crea una stringa
        // contenente le coppie chiave/valore per
        // il tipo di richiesta e la restituisce
        private System.String __Dump(System.Collections.Specialized.NameValueCollection richiesta)
        {
            System.String risultato = "";
            System.String[] chiavi = richiesta.AllKeys;
            for (System.Int32 a = 0; a < chiavi.Length; a++)
                risultato += chiavi[a] + "\r\n\t" + richiesta.Get(chiavi[a]) + "\r\n";
            return risultato;
        }
    }

    // classe FiltroRisposta, estende la classe System.IO.Stream
    //	crea un file di testo contenente i dati
    // 	inviati al client se assegnata come filtro.
    //
    //  versione riadattata del codice presente in questa pagina:
    //    http://support.microsoft.com/default.aspx?scid=kb;en-us;810205
    public class FiltroRisposta : System.IO.Stream
    {

        // output mostrato al client
        private System.IO.Stream __browser;
        // posizione stringa
        private System.Int64 __posizione;
        // file dove aggiungere le informazioni
        // testuali mostrate al client (output testuale)
        private System.IO.FileStream __fileReport;

        // costruttore pubblico,
        //   accetta uno stream, in questo caso il Filter della Response
        //   ed il nome del file per riferimento sulla quale effettuare il report
        public FiltroRisposta(System.IO.Stream browser, ref System.String fileReport)
        {
            __browser = browser;
            __fileReport = new System.IO.FileStream(
                                    fileReport, 
                                    System.IO.FileMode.OpenOrCreate, 
                                    System.IO.FileAccess.Write
                                );
        }

        // metodo pubblico,
        //    scrive una stringa sul file di report, in questo caso
        //    la variabile inizioReport inviata per riferimento dal gestore
        //    (vedere la classe InformazioniAjax ed il metodo CreaReport)
        public void ScriviInizioReport(ref System.String inizioReport)
        {
            System.Byte[] buffer = System.Text.Encoding.Default.GetBytes(inizioReport);
            __fileReport.Write(buffer, 0, buffer.Length);
        }

        // override di un metodo pubblico,
        //    invia l'output al browser e scrive lo stesso nel file inviato al costruttore
        public override void Write(System.Byte[] buffer, System.Int32 offset, System.Int32 count)
        {
            __browser.Write(buffer, 0, count);
            __fileReport.Write(buffer, 0, count);
        }

        // lista di metodi pubblici sulla quale
        // effettuare l'override per scavalcare
        // i controlli standard sul flusso dati in uscita
        public override System.Boolean CanRead
        { get { return true; } }
        public override System.Boolean CanSeek
        { get { return false; } }
        public override System.Boolean CanWrite
        { get { return false; } }
        public override System.Int64 Length
        { get { return 0; } }
        public override long Seek(System.Int64 offset, System.IO.SeekOrigin direction)
        { return 0; }
        public override void SetLength(System.Int64 length)
        { __browser.SetLength(length); }
        public override void Flush()
        { __browser.Flush(); }
        public override int Read(System.Byte[] buffer, System.Int32 offset, System.Int32 count)
        { return __browser.Read(buffer, offset, count); }
        public override System.Int64 Position
        {
            get { return __posizione; }
            set { __posizione = value; }
        }
        public override void Close()
        {
            __browser.Close();
            __fileReport.Close();
        }
    }
}